@@ -232,7 +232,7 @@ impl RegistryClient {
                 }
                 Err(err) => match err.into_kind() {
                     // The package could not be found in the remote index.
-                    ErrorKind::WrappedReqwestError(err) => match err.status() {
+                    ErrorKind::WrappedReqwestError(url, err) => match err.status() {
                         Some(StatusCode::NOT_FOUND) => {}
                         Some(StatusCode::UNAUTHORIZED) => {
                             capabilities.set_unauthorized(index.clone());
@@ -240,7 +240,7 @@ impl RegistryClient {
                         Some(StatusCode::FORBIDDEN) => {
                             capabilities.set_forbidden(index.clone());
                         }
-                        _ => return Err(ErrorKind::from(err).into()),
+                        _ => return Err(ErrorKind::WrappedReqwestError(url, err).into()),
                     },
 
                     // The package is unavailable due to a lack of connectivity.
@@ -323,7 +323,7 @@ impl RegistryClient {
             .header("Accept-Encoding", "gzip")
             .header("Accept", MediaType::accepts())
             .build()
-            .map_err(ErrorKind::from)?;
+            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
         let parse_simple_response = |response: Response| {
             async {
                 // Use the response URL, rather than the request URL, as the base for relative URLs.
@@ -347,14 +347,20 @@ impl RegistryClient {
 
                 let unarchived = match media_type {
                     MediaType::Json => {
-                        let bytes = response.bytes().await.map_err(ErrorKind::from)?;
+                        let bytes = response
+                            .bytes()
+                            .await
+                            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
                         let data: SimpleJson = serde_json::from_slice(bytes.as_ref())
                             .map_err(|err| Error::from_json_err(err, url.clone()))?;
 
                         SimpleMetadata::from_files(data.files, package_name, &url)
                     }
                     MediaType::Html => {
-                        let text = response.text().await.map_err(ErrorKind::from)?;
+                        let text = response
+                            .text()
+                            .await
+                            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
                         SimpleMetadata::from_html(&text, package_name, &url)?
                     }
                 };
@@ -547,7 +553,10 @@ impl RegistryClient {
             };
 
             let response_callback = |response: Response| async {
-                let bytes = response.bytes().await.map_err(ErrorKind::from)?;
+                let bytes = response
+                    .bytes()
+                    .await
+                    .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
 
                 info_span!("parse_metadata21")
                     .in_scope(|| ResolutionMetadata::parse_metadata(bytes.as_ref()))
@@ -563,7 +572,7 @@ impl RegistryClient {
                 .uncached_client(&url)
                 .get(url.clone())
                 .build()
-                .map_err(ErrorKind::from)?;
+                .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
             Ok(self
                 .cached_client()
                 .get_serde(req, &cache_entry, cache_control, response_callback)
@@ -616,7 +625,7 @@ impl RegistryClient {
                     http::HeaderValue::from_static("identity"),
                 )
                 .build()
-                .map_err(ErrorKind::from)?;
+                .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
 
             // Copy authorization headers from the HEAD request to subsequent requests
             let mut headers = HeaderMap::default();
@@ -694,7 +703,7 @@ impl RegistryClient {
                 reqwest::header::HeaderValue::from_static("identity"),
             )
             .build()
-            .map_err(ErrorKind::from)?;
+            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
 
         // Stream the file, searching for the METADATA.
         let read_metadata_stream = |response: Response| {
