@@ -101,7 +101,14 @@ fn generate() -> String {
     generate_command(&mut output, &uv, &mut parents);
 
     for (value, replacement) in REPLACEMENTS {
-        output = output.replace(value, replacement);
+        assert_ne!(
+            value, replacement,
+            "`value` and `replacement` must be different, but both are `{value}`"
+        );
+        let before = &output;
+        let after = output.replace(value, replacement);
+        assert_ne!(*before, after, "Could not find `{value}` in the output");
+        output = after;
     }
 
     output
