@@ -1,5 +1,5 @@
 use itertools::Either;
-use rustc_hash::FxHashSet;
+use rustc_hash::{FxHashMap, FxHashSet};
 use std::borrow::Cow;
 use std::fmt::{Display, Formatter};
 use std::ops::Deref;
@@ -11,12 +11,13 @@ use url::{ParseError, Url};
 
 use uv_pep508::{VerbatimUrl, VerbatimUrlError};
 
-use crate::Verbatim;
+use crate::{Index, Verbatim};
 
 static PYPI_URL: LazyLock<Url> = LazyLock::new(|| Url::parse("https://pypi.org/simple").unwrap());
 
-static DEFAULT_INDEX_URL: LazyLock<IndexUrl> =
-    LazyLock::new(|| IndexUrl::Pypi(VerbatimUrl::from_url(PYPI_URL.clone())));
+static DEFAULT_INDEX: LazyLock<Index> = LazyLock::new(|| {
+    Index::from_index_url(IndexUrl::Pypi(VerbatimUrl::from_url(PYPI_URL.clone())))
+});
 
 /// The URL of an index to use for fetching packages (e.g., PyPI).
 #[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
@@ -55,6 +56,15 @@ impl IndexUrl {
         }
     }
 
+    /// Convert the index URL into a [`Url`].
+    pub fn into_url(self) -> Url {
+        match self {
+            Self::Pypi(url) => url.into_url(),
+            Self::Url(url) => url.into_url(),
+            Self::Path(url) => url.into_url(),
+        }
+    }
+
     /// Return the redacted URL for the index, omitting any sensitive credentials.
     pub fn redacted(&self) -> Cow<'_, Url> {
         let url = self.url();
@@ -89,15 +99,6 @@ impl Verbatim for IndexUrl {
     }
 }
 
-impl From<FlatIndexLocation> for IndexUrl {
-    fn from(location: FlatIndexLocation) -> Self {
-        match location {
-            FlatIndexLocation::Path(url) => Self::Path(url),
-            FlatIndexLocation::Url(url) => Self::Url(url),
-        }
-    }
-}
-
 /// An error that can occur when parsing an [`IndexUrl`].
 #[derive(Error, Debug)]
 pub enum IndexUrlError {
@@ -175,152 +176,23 @@ impl Deref for IndexUrl {
     }
 }
 
-/// A directory with distributions or a URL to an HTML file with a flat listing of distributions.
-///
-/// Also known as `--find-links`.
-#[derive(Debug, Clone, Hash, Eq, PartialEq)]
-pub enum FlatIndexLocation {
-    Path(VerbatimUrl),
-    Url(VerbatimUrl),
-}
-
-#[cfg(feature = "schemars")]
-impl schemars::JsonSchema for FlatIndexLocation {
-    fn schema_name() -> String {
-        "FlatIndexLocation".to_string()
-    }
-
-    fn json_schema(_gen: &mut schemars::gen::SchemaGenerator) -> schemars::schema::Schema {
-        schemars::schema::SchemaObject {
-            instance_type: Some(schemars::schema::InstanceType::String.into()),
-            metadata: Some(Box::new(schemars::schema::Metadata {
-                description: Some("The path to a directory of distributions, or a URL to an HTML file with a flat listing of distributions.".to_string()),
-              ..schemars::schema::Metadata::default()
-            })),
-            ..schemars::schema::SchemaObject::default()
-        }
-        .into()
-    }
-}
-
-impl FlatIndexLocation {
-    /// Return the raw URL for the `--find-links` index.
-    pub fn url(&self) -> &Url {
-        match self {
-            Self::Url(url) => url.raw(),
-            Self::Path(url) => url.raw(),
-        }
-    }
-
-    /// Return the redacted URL for the `--find-links` index, omitting any sensitive credentials.
-    pub fn redacted(&self) -> Cow<'_, Url> {
-        let url = self.url();
-        if url.username().is_empty() && url.password().is_none() {
-            Cow::Borrowed(url)
-        } else {
-            let mut url = url.clone();
-            let _ = url.set_username("");
-            let _ = url.set_password(None);
-            Cow::Owned(url)
-        }
-    }
-}
-
-impl Display for FlatIndexLocation {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            Self::Url(url) => Display::fmt(url, f),
-            Self::Path(url) => Display::fmt(url, f),
-        }
-    }
-}
-
-impl Verbatim for FlatIndexLocation {
-    fn verbatim(&self) -> Cow<'_, str> {
-        match self {
-            Self::Url(url) => url.verbatim(),
-            Self::Path(url) => url.verbatim(),
-        }
-    }
-}
-
-impl FromStr for FlatIndexLocation {
-    type Err = IndexUrlError;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let url = if Path::new(s).exists() {
-            VerbatimUrl::from_absolute_path(std::path::absolute(s)?)?
-        } else {
-            VerbatimUrl::parse_url(s)?
-        };
-        Ok(Self::from(url.with_given(s)))
-    }
-}
-
-impl serde::ser::Serialize for FlatIndexLocation {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: serde::ser::Serializer,
-    {
-        self.to_string().serialize(serializer)
-    }
-}
-
-impl<'de> serde::de::Deserialize<'de> for FlatIndexLocation {
-    fn deserialize<D>(deserializer: D) -> Result<FlatIndexLocation, D::Error>
-    where
-        D: serde::de::Deserializer<'de>,
-    {
-        let s = String::deserialize(deserializer)?;
-        FlatIndexLocation::from_str(&s).map_err(serde::de::Error::custom)
-    }
-}
-
-impl From<VerbatimUrl> for FlatIndexLocation {
-    fn from(url: VerbatimUrl) -> Self {
-        if url.scheme() == "file" {
-            Self::Path(url)
-        } else {
-            Self::Url(url)
-        }
-    }
-}
-
 /// The index locations to use for fetching packages. By default, uses the PyPI index.
 ///
-/// From a pip perspective, this type merges `--index-url`, `--extra-index-url`, and `--find-links`.
-#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
+/// This type merges the legacy `--index-url`, `--extra-index-url`, and `--find-links` options,
+/// along with the uv-specific `--index` and `--default-index`.
+#[derive(Default, Debug, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
 #[serde(rename_all = "kebab-case", deny_unknown_fields)]
 pub struct IndexLocations {
-    index: Option<IndexUrl>,
-    extra_index: Vec<IndexUrl>,
-    flat_index: Vec<FlatIndexLocation>,
+    indexes: Vec<Index>,
+    flat_index: Vec<Index>,
     no_index: bool,
 }
 
-impl Default for IndexLocations {
-    /// By default, use the `PyPI` index.
-    fn default() -> Self {
-        Self {
-            index: Some(DEFAULT_INDEX_URL.clone()),
-            extra_index: Vec::new(),
-            flat_index: Vec::new(),
-            no_index: false,
-        }
-    }
-}
-
 impl IndexLocations {
     /// Determine the index URLs to use for fetching packages.
-    pub fn new(
-        index: Option<IndexUrl>,
-        extra_index: Vec<IndexUrl>,
-        flat_index: Vec<FlatIndexLocation>,
-        no_index: bool,
-    ) -> Self {
+    pub fn new(indexes: Vec<Index>, flat_index: Vec<Index>, no_index: bool) -> Self {
         Self {
-            index,
-            extra_index,
+            indexes,
             flat_index,
             no_index,
         }
@@ -333,16 +205,9 @@ impl IndexLocations {
     ///
     /// If the current index location has an `index` set, it will be preserved.
     #[must_use]
-    pub fn combine(
-        self,
-        index: Option<IndexUrl>,
-        extra_index: Vec<IndexUrl>,
-        flat_index: Vec<FlatIndexLocation>,
-        no_index: bool,
-    ) -> Self {
+    pub fn combine(self, indexes: Vec<Index>, flat_index: Vec<Index>, no_index: bool) -> Self {
         Self {
-            index: self.index.or(index),
-            extra_index: self.extra_index.into_iter().chain(extra_index).collect(),
+            indexes: self.indexes.into_iter().chain(indexes).collect(),
             flat_index: self.flat_index.into_iter().chain(flat_index).collect(),
             no_index: self.no_index || no_index,
         }
@@ -351,51 +216,73 @@ impl IndexLocations {
     /// Returns `true` if no index configuration is set, i.e., the [`IndexLocations`] matches the
     /// default configuration.
     pub fn is_none(&self) -> bool {
-        self.index.is_none()
-            && self.extra_index.is_empty()
-            && self.flat_index.is_empty()
-            && !self.no_index
+        *self == Self::default()
     }
 }
 
 impl<'a> IndexLocations {
-    /// Return the primary [`IndexUrl`] entry.
+    /// Return the default [`Index`] entry.
     ///
     /// If `--no-index` is set, return `None`.
     ///
     /// If no index is provided, use the `PyPI` index.
-    pub fn index(&'a self) -> Option<&'a IndexUrl> {
+    pub fn default_index(&'a self) -> Option<&'a Index> {
         if self.no_index {
             None
         } else {
-            match self.index.as_ref() {
-                Some(index) => Some(index),
-                None => Some(&DEFAULT_INDEX_URL),
-            }
+            let mut seen = FxHashSet::default();
+            self.indexes
+                .iter()
+                .filter(move |index| index.name.as_ref().map_or(true, |name| seen.insert(name)))
+                .find(|index| index.default && !index.explicit)
+                .or_else(|| Some(&DEFAULT_INDEX))
         }
     }
 
-    /// Return an iterator over the extra [`IndexUrl`] entries.
-    pub fn extra_index(&'a self) -> impl Iterator<Item = &'a IndexUrl> + 'a {
+    /// Return an iterator over the implicit [`Index`] entries.
+    pub fn implicit_indexes(&'a self) -> impl Iterator<Item = &'a Index> + 'a {
         if self.no_index {
             Either::Left(std::iter::empty())
         } else {
-            Either::Right(self.extra_index.iter())
+            let mut seen = FxHashSet::default();
+            Either::Right(
+                self.indexes
+                    .iter()
+                    .filter(move |index| index.name.as_ref().map_or(true, |name| seen.insert(name)))
+                    .filter(|index| !(index.default || index.explicit)),
+            )
         }
     }
 
-    /// Return an iterator over all [`IndexUrl`] entries in order.
+    /// Return an iterator over the explicit [`Index`] entries.
+    pub fn explicit_indexes(&'a self) -> impl Iterator<Item = &'a Index> + 'a {
+        if self.no_index {
+            Either::Left(std::iter::empty())
+        } else {
+            let mut seen = FxHashSet::default();
+            Either::Right(
+                self.indexes
+                    .iter()
+                    .filter(move |index| index.name.as_ref().map_or(true, |name| seen.insert(name)))
+                    .filter(|index| index.explicit),
+            )
+        }
+    }
+
+    /// Return an iterator over all [`Index`] entries in order.
     ///
-    /// Prioritizes the extra indexes over the main index.
+    /// Explicit indexes are excluded.
+    ///
+    /// Prioritizes the extra indexes over the default index.
     ///
     /// If `no_index` was enabled, then this always returns an empty
     /// iterator.
-    pub fn indexes(&'a self) -> impl Iterator<Item = &'a IndexUrl> + 'a {
-        self.extra_index().chain(self.index())
+    pub fn indexes(&'a self) -> impl Iterator<Item = &'a Index> + 'a {
+        self.implicit_indexes().chain(self.default_index())
     }
 
     /// Return an iterator over the [`FlatIndexLocation`] entries.
-    pub fn flat_index(&'a self) -> impl Iterator<Item = &'a FlatIndexLocation> + 'a {
+    pub fn flat_indexes(&'a self) -> impl Iterator<Item = &'a Index> + 'a {
         self.flat_index.iter()
     }
 
@@ -407,111 +294,181 @@ impl<'a> IndexLocations {
     /// Clone the index locations into a [`IndexUrls`] instance.
     pub fn index_urls(&'a self) -> IndexUrls {
         IndexUrls {
-            index: self.index.clone(),
-            extra_index: self.extra_index.clone(),
+            indexes: self.indexes.clone(),
             no_index: self.no_index,
         }
     }
 
-    /// Return an iterator over all [`Url`] entries.
-    pub fn urls(&'a self) -> impl Iterator<Item = &'a Url> + 'a {
-        self.indexes()
-            .map(IndexUrl::url)
-            .chain(self.flat_index.iter().filter_map(|index| match index {
-                FlatIndexLocation::Path(_) => None,
-                FlatIndexLocation::Url(url) => Some(url.raw()),
-            }))
+    /// Return a vector containing all allowed [`Index`] entries.
+    ///
+    /// This includes explicit indexes, implicit indexes, flat indexes, and the default index.
+    ///
+    /// The indexes will be returned in the order in which they were defined, such that the
+    /// last-defined index is the last item in the vector.
+    pub fn allowed_indexes(&'a self) -> Vec<&'a Index> {
+        if self.no_index {
+            self.flat_index.iter().rev().collect()
+        } else {
+            let mut indexes = vec![];
+
+            let mut seen = FxHashSet::default();
+            let mut default = false;
+            for index in {
+                self.indexes
+                    .iter()
+                    .chain(self.flat_index.iter())
+                    .filter(move |index| index.name.as_ref().map_or(true, |name| seen.insert(name)))
+            } {
+                if index.default && !index.explicit {
+                    if default {
+                        continue;
+                    }
+                    default = true;
+                }
+                indexes.push(index);
+            }
+            if !default {
+                indexes.push(&*DEFAULT_INDEX);
+            }
+
+            indexes.reverse();
+            indexes
+        }
     }
 }
 
 /// The index URLs to use for fetching packages.
 ///
-/// From a pip perspective, this type merges `--index-url` and `--extra-index-url`.
-#[derive(Debug, Clone)]
+/// This type merges the legacy `--index-url` and `--extra-index-url` options, along with the
+/// uv-specific `--index` and `--default-index`.
+#[derive(Default, Debug, Clone, PartialEq, Eq)]
 pub struct IndexUrls {
-    index: Option<IndexUrl>,
-    extra_index: Vec<IndexUrl>,
+    indexes: Vec<Index>,
     no_index: bool,
 }
 
-impl Default for IndexUrls {
-    /// By default, use the `PyPI` index.
-    fn default() -> Self {
-        Self {
-            index: Some(DEFAULT_INDEX_URL.clone()),
-            extra_index: Vec::new(),
-            no_index: false,
-        }
-    }
-}
-
 impl<'a> IndexUrls {
-    /// Return the fallback [`IndexUrl`] entry.
+    /// Return the default [`Index`] entry.
     ///
     /// If `--no-index` is set, return `None`.
     ///
     /// If no index is provided, use the `PyPI` index.
-    fn index(&'a self) -> Option<&'a IndexUrl> {
+    fn default_index(&'a self) -> Option<&'a Index> {
         if self.no_index {
             None
         } else {
-            match self.index.as_ref() {
-                Some(index) => Some(index),
-                None => Some(&DEFAULT_INDEX_URL),
-            }
+            let mut seen = FxHashSet::default();
+            self.indexes
+                .iter()
+                .filter(move |index| index.name.as_ref().map_or(true, |name| seen.insert(name)))
+                .find(|index| index.default && !index.explicit)
+                .or_else(|| Some(&DEFAULT_INDEX))
         }
     }
 
-    /// Return an iterator over the extra [`IndexUrl`] entries.
-    fn extra_index(&'a self) -> impl Iterator<Item = &'a IndexUrl> + 'a {
+    /// Return an iterator over the implicit [`Index`] entries.
+    fn implicit_indexes(&'a self) -> impl Iterator<Item = &'a Index> + 'a {
         if self.no_index {
             Either::Left(std::iter::empty())
         } else {
-            Either::Right(self.extra_index.iter())
+            let mut seen = FxHashSet::default();
+            Either::Right(
+                self.indexes
+                    .iter()
+                    .filter(move |index| index.name.as_ref().map_or(true, |name| seen.insert(name)))
+                    .filter(|index| !(index.default || index.explicit)),
+            )
         }
     }
 
     /// Return an iterator over all [`IndexUrl`] entries in order.
     ///
-    /// Prioritizes the extra indexes over the main index.
+    /// Prioritizes the `[tool.uv.index]` definitions over the `--extra-index-url` definitions
+    /// over the `--index-url` definition.
     ///
     /// If `no_index` was enabled, then this always returns an empty
     /// iterator.
-    pub fn indexes(&'a self) -> impl Iterator<Item = &'a IndexUrl> + 'a {
-        self.extra_index().chain(self.index())
+    pub fn indexes(&'a self) -> impl Iterator<Item = &'a Index> + 'a {
+        self.implicit_indexes().chain(self.default_index())
     }
 }
 
-impl From<IndexLocations> for IndexUrls {
-    fn from(locations: IndexLocations) -> Self {
-        Self {
-            index: locations.index,
-            extra_index: locations.extra_index,
-            no_index: locations.no_index,
-        }
+bitflags::bitflags! {
+    #[derive(Debug, Copy, Clone)]
+    struct Flags: u8 {
+        /// Whether the index supports range requests.
+        const NO_RANGE_REQUESTS = 1;
+        /// Whether the index returned a `401 Unauthorized` status code.
+        const UNAUTHORIZED      = 1 << 2;
+        /// Whether the index returned a `403 Forbidden` status code.
+        const FORBIDDEN         = 1 << 1;
     }
 }
 
 /// A map of [`IndexUrl`]s to their capabilities.
 ///
-/// For now, we only support a single capability (range requests), and we only store an index if
-/// it _doesn't_ support range requests. The benefit is that the map is almost always empty, so
-/// validating capabilities is extremely cheap.
+/// We only store indexes that lack capabilities (i.e., don't support range requests, aren't
+/// authorized). The benefit is that the map is almost always empty, so validating capabilities is
+/// extremely cheap.
 #[derive(Debug, Default, Clone)]
-pub struct IndexCapabilities(Arc<RwLock<FxHashSet<IndexUrl>>>);
+pub struct IndexCapabilities(Arc<RwLock<FxHashMap<IndexUrl, Flags>>>);
 
 impl IndexCapabilities {
     /// Returns `true` if the given [`IndexUrl`] supports range requests.
     pub fn supports_range_requests(&self, index_url: &IndexUrl) -> bool {
-        !self.0.read().unwrap().contains(index_url)
+        !self
+            .0
+            .read()
+            .unwrap()
+            .get(index_url)
+            .is_some_and(|flags| flags.intersects(Flags::NO_RANGE_REQUESTS))
     }
 
     /// Mark an [`IndexUrl`] as not supporting range requests.
-    pub fn set_supports_range_requests(&self, index_url: IndexUrl, supports: bool) {
-        if supports {
-            self.0.write().unwrap().remove(&index_url);
-        } else {
-            self.0.write().unwrap().insert(index_url);
-        }
+    pub fn set_no_range_requests(&self, index_url: IndexUrl) {
+        self.0
+            .write()
+            .unwrap()
+            .entry(index_url)
+            .or_insert(Flags::empty())
+            .insert(Flags::NO_RANGE_REQUESTS);
+    }
+
+    /// Returns `true` if the given [`IndexUrl`] returns a `401 Unauthorized` status code.
+    pub fn unauthorized(&self, index_url: &IndexUrl) -> bool {
+        self.0
+            .read()
+            .unwrap()
+            .get(index_url)
+            .is_some_and(|flags| flags.intersects(Flags::UNAUTHORIZED))
+    }
+
+    /// Mark an [`IndexUrl`] as returning a `401 Unauthorized` status code.
+    pub fn set_unauthorized(&self, index_url: IndexUrl) {
+        self.0
+            .write()
+            .unwrap()
+            .entry(index_url)
+            .or_insert(Flags::empty())
+            .insert(Flags::UNAUTHORIZED);
+    }
+
+    /// Returns `true` if the given [`IndexUrl`] returns a `403 Forbidden` status code.
+    pub fn forbidden(&self, index_url: &IndexUrl) -> bool {
+        self.0
+            .read()
+            .unwrap()
+            .get(index_url)
+            .is_some_and(|flags| flags.intersects(Flags::FORBIDDEN))
+    }
+
+    /// Mark an [`IndexUrl`] as returning a `403 Forbidden` status code.
+    pub fn set_forbidden(&self, index_url: IndexUrl) {
+        self.0
+            .write()
+            .unwrap()
+            .entry(index_url)
+            .or_insert(Flags::empty())
+            .insert(Flags::FORBIDDEN);
     }
 }
