@@ -8,6 +8,7 @@ use uv_distribution_types::IndexLocations;
 use uv_normalize::{ExtraName, GroupName, PackageName};
 use uv_pep440::{Version, VersionSpecifiers};
 use uv_pypi_types::{HashDigest, ResolutionMetadata};
+use uv_workspace::dependency_groups::DependencyGroupError;
 use uv_workspace::WorkspaceError;
 
 pub use crate::metadata::lowering::LoweredRequirement;
@@ -21,10 +22,12 @@ mod requires_dist;
 pub enum MetadataError {
     #[error(transparent)]
     Workspace(#[from] WorkspaceError),
-    #[error("Failed to parse entry for: `{0}`")]
-    LoweringError(PackageName, #[source] LoweringError),
     #[error(transparent)]
-    Lower(#[from] LoweringError),
+    DependencyGroup(#[from] DependencyGroupError),
+    #[error("Failed to parse entry: `{0}`")]
+    LoweringError(PackageName, #[source] Box<LoweringError>),
+    #[error("Failed to parse entry in group `{0}`: `{1}`")]
+    GroupLoweringError(GroupName, PackageName, #[source] Box<LoweringError>),
 }
 
 #[derive(Debug, Clone)]
@@ -36,7 +39,7 @@ pub struct Metadata {
     pub requires_dist: Vec<uv_pypi_types::Requirement>,
     pub requires_python: Option<VersionSpecifiers>,
     pub provides_extras: Vec<ExtraName>,
-    pub dev_dependencies: BTreeMap<GroupName, Vec<uv_pypi_types::Requirement>>,
+    pub dependency_groups: BTreeMap<GroupName, Vec<uv_pypi_types::Requirement>>,
 }
 
 impl Metadata {
@@ -53,7 +56,7 @@ impl Metadata {
                 .collect(),
             requires_python: metadata.requires_python,
             provides_extras: metadata.provides_extras,
-            dev_dependencies: BTreeMap::default(),
+            dependency_groups: BTreeMap::default(),
         }
     }
 
@@ -71,7 +74,7 @@ impl Metadata {
             name,
             requires_dist,
             provides_extras,
-            dev_dependencies,
+            dependency_groups,
         } = RequiresDist::from_project_maybe_workspace(
             uv_pypi_types::RequiresDist {
                 name: metadata.name,
@@ -92,7 +95,7 @@ impl Metadata {
             requires_dist,
             requires_python: metadata.requires_python,
             provides_extras,
-            dev_dependencies,
+            dependency_groups,
         })
     }
 }
