@@ -1,5 +1,9 @@
 //! Resolve the current [`ProjectWorkspace`] or [`Workspace`].
 
+use crate::dependency_groups::{DependencyGroupError, FlatDependencyGroups};
+use crate::pyproject::{
+    Project, PyProjectToml, PyprojectTomlError, Sources, ToolUvSources, ToolUvWorkspace,
+};
 use either::Either;
 use glob::{glob, GlobError, PatternError};
 use rustc_hash::FxHashSet;
@@ -14,10 +18,6 @@ use uv_pypi_types::{Requirement, RequirementSource, SupportedEnvironments, Verba
 use uv_static::EnvVars;
 use uv_warnings::{warn_user, warn_user_once};
 
-use crate::pyproject::{
-    Project, PyProjectToml, PyprojectTomlError, Sources, ToolUvSources, ToolUvWorkspace,
-};
-
 #[derive(thiserror::Error, Debug)]
 pub enum WorkspaceError {
     // Workspace structure errors.
@@ -305,7 +305,7 @@ impl Workspace {
     /// `pyproject.toml`.
     ///
     /// Otherwise, returns an empty list.
-    pub fn non_project_requirements(&self) -> impl Iterator<Item = Requirement> + '_ {
+    pub fn non_project_requirements(&self) -> Result<Vec<Requirement>, DependencyGroupError> {
         if self
             .packages
             .values()
@@ -313,25 +313,46 @@ impl Workspace {
         {
             // If the workspace has an explicit root, the root is a member, so we don't need to
             // include any root-only requirements.
-            Either::Left(std::iter::empty())
+            Ok(Vec::new())
         } else {
-            // Otherwise, return the dev dependencies in the non-project workspace root.
-            Either::Right(
-                self.pyproject_toml
-                    .tool
-                    .as_ref()
-                    .and_then(|tool| tool.uv.as_ref())
-                    .and_then(|uv| uv.dev_dependencies.as_ref())
-                    .into_iter()
-                    .flatten()
-                    .map(|requirement| {
-                        Requirement::from(
-                            requirement
-                                .clone()
-                                .with_origin(RequirementOrigin::Workspace),
-                        )
-                    }),
-            )
+            // Otherwise, return the dependency groups in the non-project workspace root.
+            // First, collect `tool.uv.dev_dependencies`
+            let dev_dependencies = self
+                .pyproject_toml
+                .tool
+                .as_ref()
+                .and_then(|tool| tool.uv.as_ref())
+                .and_then(|uv| uv.dev_dependencies.as_ref());
+
+            // Then, collect `dependency-groups`
+            let dependency_groups = self
+                .pyproject_toml
+                .dependency_groups
+                .iter()
+                .flatten()
+                .collect::<BTreeMap<_, _>>();
+
+            // Resolve any `include-group` entries in `dependency-groups`.
+            let dependency_groups =
+                FlatDependencyGroups::from_dependency_groups(&dependency_groups)?;
+
+            // Concatenate the two sets of requirements.
+            let dev_dependencies = dependency_groups
+                .into_iter()
+                .flat_map(|(_, requirements)| requirements)
+                .map(|requirement| {
+                    Requirement::from(requirement.with_origin(RequirementOrigin::Workspace))
+                })
+                .chain(dev_dependencies.into_iter().flatten().map(|requirement| {
+                    Requirement::from(
+                        requirement
+                            .clone()
+                            .with_origin(RequirementOrigin::Workspace),
+                    )
+                }))
+                .collect();
+
+            Ok(dev_dependencies)
         }
     }
 
@@ -1434,7 +1455,7 @@ impl VirtualProject {
 }
 
 /// A target that can be installed.
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Copy, Clone)]
 pub enum InstallTarget<'env> {
     /// A project (which could be a workspace root or member).
     Project(&'env ProjectWorkspace),
@@ -1468,38 +1489,62 @@ impl<'env> InstallTarget<'env> {
         }
     }
 
-    /// Return the [`InstallTarget`] dependencies for the given group name.
+    /// Return the [`InstallTarget`] dependency groups.
     ///
     /// Returns dependencies that apply to the workspace root, but not any of its members. As such,
     /// only returns a non-empty iterator for virtual workspaces, which can include dev dependencies
     /// on the virtual root.
-    pub fn group(
+    pub fn groups(
         &self,
-        name: &GroupName,
-    ) -> impl Iterator<Item = &uv_pep508::Requirement<VerbatimParsedUrl>> {
+    ) -> Result<
+        BTreeMap<GroupName, Vec<uv_pep508::Requirement<VerbatimParsedUrl>>>,
+        DependencyGroupError,
+    > {
         match self {
-            Self::Project(_) | Self::FrozenMember(..) => {
-                // For projects, dev dependencies are attached to the members.
-                Either::Left(std::iter::empty())
-            }
+            Self::Project(_) | Self::FrozenMember(..) => Ok(BTreeMap::new()),
             Self::NonProject(workspace) => {
-                // For non-projects, we might have dev dependencies that are attached to the
-                // workspace root (which isn't a member).
-                if name == &*DEV_DEPENDENCIES {
-                    Either::Right(
-                        workspace
-                            .pyproject_toml
-                            .tool
-                            .as_ref()
-                            .and_then(|tool| tool.uv.as_ref())
-                            .and_then(|uv| uv.dev_dependencies.as_ref())
-                            .map(|dev| dev.iter())
-                            .into_iter()
-                            .flatten(),
-                    )
-                } else {
-                    Either::Left(std::iter::empty())
+                // For non-projects, we might have `dependency-groups` or `tool.uv.dev-dependencies`
+                // that are attached to the workspace root (which isn't a member).
+
+                // First, collect `tool.uv.dev_dependencies`
+                let dev_dependencies = workspace
+                    .pyproject_toml()
+                    .tool
+                    .as_ref()
+                    .and_then(|tool| tool.uv.as_ref())
+                    .and_then(|uv| uv.dev_dependencies.as_ref());
+
+                // Then, collect `dependency-groups`
+                let dependency_groups = workspace
+                    .pyproject_toml()
+                    .dependency_groups
+                    .iter()
+                    .flatten()
+                    .collect::<BTreeMap<_, _>>();
+
+                // Merge any overlapping groups.
+                let mut map = BTreeMap::new();
+                for (name, dependencies) in
+                    FlatDependencyGroups::from_dependency_groups(&dependency_groups)?
+                        .into_iter()
+                        .chain(
+                            // Only add the `dev` group if `dev-dependencies` is defined.
+                            dev_dependencies.into_iter().map(|requirements| {
+                                (DEV_DEPENDENCIES.clone(), requirements.clone())
+                            }),
+                        )
+                {
+                    match map.entry(name) {
+                        std::collections::btree_map::Entry::Vacant(entry) => {
+                            entry.insert(dependencies);
+                        }
+                        std::collections::btree_map::Entry::Occupied(mut entry) => {
+                            entry.get_mut().extend(dependencies);
+                        }
+                    }
                 }
+
+                Ok(map)
             }
         }
     }
