@@ -187,14 +187,19 @@ async fn build_impl(
             }
         };
 
-        let project = workspace
+        let package = workspace
             .packages()
             .get(package)
-            .ok_or_else(|| anyhow::anyhow!("Package `{}` not found in workspace", package))?
-            .root();
+            .ok_or_else(|| anyhow::anyhow!("Package `{package}` not found in workspace"))?;
+
+        if !package.pyproject_toml().is_package() {
+            let name = &package.project().name;
+            let pyproject_toml = package.root().join("pyproject.toml");
+            return Err(anyhow::anyhow!("Package `{}` is missing a `{}`. For example, to build with `{}`, add the following to `{}`:\n```toml\n[build-system]\nrequires = [\"setuptools\"]\nbuild-backend = \"setuptools.build_meta\"\n```", name.cyan(), "build-system".green(), "setuptools".cyan(), pyproject_toml.user_display().cyan()));
+        }
 
         vec![AnnotatedSource::from(Source::Directory(Cow::Borrowed(
-            project,
+            package.root(),
         )))]
     } else if all {
         if matches!(src, Source::File(_)) {
@@ -211,6 +216,10 @@ async fn build_impl(
             }
         };
 
+        if workspace.packages().is_empty() {
+            return Err(anyhow::anyhow!("No packages found in workspace"));
+        }
+
         let packages: Vec<_> = workspace
             .packages()
             .values()
@@ -222,7 +231,10 @@ async fn build_impl(
             .collect();
 
         if packages.is_empty() {
-            return Err(anyhow::anyhow!("No packages found in workspace"));
+            let member = workspace.packages().values().next().unwrap();
+            let name = &member.project().name;
+            let pyproject_toml = member.root().join("pyproject.toml");
+            return Err(anyhow::anyhow!("Workspace does contain any buildable packages. For example, to build `{}` with `{}`, add a `{}` to `{}`:\n```toml\n[build-system]\nrequires = [\"setuptools\"]\nbuild-backend = \"setuptools.build_meta\"\n```", name.cyan(), "setuptools".cyan(), "build-system".green(), pyproject_toml.user_display().cyan()));
         }
 
         packages
