@@ -6,9 +6,10 @@ use owo_colors::OwoColorize;
 use uv_cache::Cache;
 use uv_client::Connectivity;
 use uv_configuration::{
-    Concurrency, DevMode, EditableMode, ExtrasSpecification, InstallOptions, LowerBound,
+    Concurrency, DevGroupsManifest, EditableMode, ExtrasSpecification, InstallOptions, LowerBound,
 };
 use uv_fs::Simplified;
+use uv_normalize::DEV_DEPENDENCIES;
 use uv_pep508::PackageName;
 use uv_python::{PythonDownloads, PythonPreference, PythonRequest};
 use uv_scripts::Pep723Script;
@@ -19,6 +20,7 @@ use uv_workspace::{DiscoveryOptions, InstallTarget, VirtualProject, Workspace};
 
 use crate::commands::pip::loggers::{DefaultInstallLogger, DefaultResolveLogger};
 use crate::commands::pip::operations::Modifications;
+use crate::commands::project::default_dependency_groups;
 use crate::commands::{project, ExitStatus, SharedState};
 use crate::printer::Printer;
 use crate::settings::ResolverInstallerSettings;
@@ -105,16 +107,18 @@ pub(crate) async fn remove(
                 }
             }
             DependencyType::Dev => {
-                let deps = toml.remove_dev_dependency(&package)?;
-                if deps.is_empty() {
+                let dev_deps = toml.remove_dev_dependency(&package)?;
+                let group_deps =
+                    toml.remove_dependency_group_requirement(&package, &DEV_DEPENDENCIES)?;
+                if dev_deps.is_empty() && group_deps.is_empty() {
                     warn_if_present(&package, &toml);
                     anyhow::bail!(
-                        "The dependency `{package}` could not be found in `dev-dependencies`"
+                        "The dependency `{package}` could not be found in `dev-dependencies` or `dependency-groups.dev`"
                     );
                 }
             }
-            DependencyType::Optional(ref group) => {
-                let deps = toml.remove_optional_dependency(&package, group)?;
+            DependencyType::Optional(ref extra) => {
+                let deps = toml.remove_optional_dependency(&package, extra)?;
                 if deps.is_empty() {
                     warn_if_present(&package, &toml);
                     anyhow::bail!(
@@ -122,6 +126,27 @@ pub(crate) async fn remove(
                     );
                 }
             }
+            DependencyType::Group(ref group) => {
+                if group == &*DEV_DEPENDENCIES {
+                    let dev_deps = toml.remove_dev_dependency(&package)?;
+                    let group_deps =
+                        toml.remove_dependency_group_requirement(&package, &DEV_DEPENDENCIES)?;
+                    if dev_deps.is_empty() && group_deps.is_empty() {
+                        warn_if_present(&package, &toml);
+                        anyhow::bail!(
+                            "The dependency `{package}` could not be found in `dev-dependencies` or `dependency-groups.dev`"
+                        );
+                    }
+                } else {
+                    let deps = toml.remove_dependency_group_requirement(&package, group)?;
+                    if deps.is_empty() {
+                        warn_if_present(&package, &toml);
+                        anyhow::bail!(
+                            "The dependency `{package}` could not be found in `dependency-groups`"
+                        );
+                    }
+                }
+            }
         }
     }
 
@@ -175,6 +200,7 @@ pub(crate) async fn remove(
     let lock = project::lock::do_safe_lock(
         locked,
         frozen,
+        false,
         project.workspace(),
         venv.interpreter(),
         settings.as_ref().into(),
@@ -196,16 +222,18 @@ pub(crate) async fn remove(
 
     // Perform a full sync, because we don't know what exactly is affected by the removal.
     // TODO(ibraheem): Should we accept CLI overrides for this? Should we even sync here?
-    let dev = DevMode::Include;
     let extras = ExtrasSpecification::All;
     let install_options = InstallOptions::default();
 
+    // Determine the default groups to include.
+    let defaults = default_dependency_groups(project.pyproject_toml())?;
+
     project::sync::do_sync(
         InstallTarget::from(&project),
         &venv,
         &lock,
         &extras,
-        dev,
+        &DevGroupsManifest::from_defaults(defaults),
         EditableMode::Editable,
         install_options,
         Modifications::Exact,
@@ -249,6 +277,11 @@ fn warn_if_present(name: &PackageName, pyproject: &PyProjectTomlMut) {
                     "`{name}` is an optional dependency; try calling `uv remove --optional {group}`",
                 );
             }
+            DependencyType::Group(group) => {
+                warn_user!(
+                    "`{name}` is in the `{group}` group; try calling `uv remove --group {group}`",
+                );
+            }
         }
     }
 }
