@@ -5,7 +5,7 @@ fn help() {
     let context = TestContext::new_with_versions(&[]);
 
     // The `uv help` command should show the long help message
-    uv_snapshot!(context.filters(), context.help(), @r#"
+    uv_snapshot!(context.filters(), context.help(), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -29,6 +29,7 @@ fn help() {
       build                      Build Python packages into source distributions and wheels
       publish                    Upload distributions to an index
       cache                      Manage uv's cache
+      self                       Manage the uv executable
       version                    Display uv's version
       generate-shell-completion  Generate shell completion
       help                       Display documentation for a command
@@ -67,14 +68,14 @@ fn help() {
 
 
     ----- stderr -----
-    "#);
+    "###);
 }
 
 #[test]
 fn help_flag() {
     let context = TestContext::new_with_versions(&[]);
 
-    uv_snapshot!(context.filters(), context.command().arg("--help"), @r#"
+    uv_snapshot!(context.filters(), context.command().arg("--help"), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -98,6 +99,7 @@ fn help_flag() {
       build    Build Python packages into source distributions and wheels
       publish  Upload distributions to an index
       cache    Manage uv's cache
+      self     Manage the uv executable
       version  Display uv's version
       help     Display documentation for a command
 
@@ -134,14 +136,14 @@ fn help_flag() {
     Use `uv help` for more details.
 
     ----- stderr -----
-    "#);
+    "###);
 }
 
 #[test]
 fn help_short_flag() {
     let context = TestContext::new_with_versions(&[]);
 
-    uv_snapshot!(context.filters(), context.command().arg("-h"), @r#"
+    uv_snapshot!(context.filters(), context.command().arg("-h"), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -165,6 +167,7 @@ fn help_short_flag() {
       build    Build Python packages into source distributions and wheels
       publish  Upload distributions to an index
       cache    Manage uv's cache
+      self     Manage the uv executable
       version  Display uv's version
       help     Display documentation for a command
 
@@ -201,7 +204,7 @@ fn help_short_flag() {
     Use `uv help` for more details.
 
     ----- stderr -----
-    "#);
+    "###);
 }
 
 #[test]
@@ -672,7 +675,7 @@ fn help_flag_subsubcommand() {
 fn help_unknown_subcommand() {
     let context = TestContext::new_with_versions(&[]);
 
-    uv_snapshot!(context.filters(), context.help().arg("foobar"), @r#"
+    uv_snapshot!(context.filters(), context.help().arg("foobar"), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -694,11 +697,12 @@ fn help_unknown_subcommand() {
         build
         publish
         cache
+        self
         version
         generate-shell-completion
-    "#);
+    "###);
 
-    uv_snapshot!(context.filters(), context.help().arg("foo").arg("bar"), @r#"
+    uv_snapshot!(context.filters(), context.help().arg("foo").arg("bar"), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -720,9 +724,10 @@ fn help_unknown_subcommand() {
         build
         publish
         cache
+        self
         version
         generate-shell-completion
-    "#);
+    "###);
 }
 
 #[test]
@@ -749,7 +754,7 @@ fn help_unknown_subsubcommand() {
 fn help_with_global_option() {
     let context = TestContext::new_with_versions(&[]);
 
-    uv_snapshot!(context.filters(), context.help().arg("--no-cache"), @r#"
+    uv_snapshot!(context.filters(), context.help().arg("--no-cache"), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -773,6 +778,7 @@ fn help_with_global_option() {
       build                      Build Python packages into source distributions and wheels
       publish                    Upload distributions to an index
       cache                      Manage uv's cache
+      self                       Manage the uv executable
       version                    Display uv's version
       generate-shell-completion  Generate shell completion
       help                       Display documentation for a command
@@ -811,7 +817,7 @@ fn help_with_global_option() {
 
 
     ----- stderr -----
-    "#);
+    "###);
 }
 
 #[test]
@@ -853,7 +859,7 @@ fn help_with_no_pager() {
 
     // We can't really test whether the --no-pager option works with a snapshot test.
     // It's still nice to have a test for the option to confirm the option exists.
-    uv_snapshot!(context.filters(), context.help().arg("--no-pager"), @r#"
+    uv_snapshot!(context.filters(), context.help().arg("--no-pager"), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -877,6 +883,7 @@ fn help_with_no_pager() {
       build                      Build Python packages into source distributions and wheels
       publish                    Upload distributions to an index
       cache                      Manage uv's cache
+      self                       Manage the uv executable
       version                    Display uv's version
       generate-shell-completion  Generate shell completion
       help                       Display documentation for a command
@@ -915,5 +922,5 @@ fn help_with_no_pager() {
 
 
     ----- stderr -----
-    "#);
+    "###);
 }
