@@ -4,6 +4,7 @@ use anyhow::Result;
 use assert_cmd::assert::OutputAssertExt;
 use assert_fs::{fixture::ChildPath, prelude::*};
 use indoc::indoc;
+use insta::assert_snapshot;
 use predicates::str::contains;
 use std::path::Path;
 
@@ -415,6 +416,30 @@ fn run_pep723_script() -> Result<()> {
        "#
     })?;
 
+    // Running a script with `--group` should warn.
+    uv_snapshot!(context.filters(), context.run().arg("--group").arg("foo").arg("main.py"), @r###"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+
+    ----- stderr -----
+    Reading inline script metadata from `main.py`
+      × No solution found when resolving script dependencies:
+      ╰─▶ Because there are no versions of add and you require add, we can conclude that your requirements are unsatisfiable.
+    "###);
+
+    // If the script can't be resolved, we should reference the script.
+    let test_script = context.temp_dir.child("main.py");
+    test_script.write_str(indoc! { r#"
+        # /// script
+        # requires-python = ">=3.11"
+        # dependencies = [
+        #   "add",
+        # ]
+        # ///
+       "#
+    })?;
+
     uv_snapshot!(context.filters(), context.run().arg("--no-project").arg("main.py"), @r###"
     success: false
     exit_code: 1
@@ -924,6 +949,161 @@ fn run_with_editable() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn run_group() -> Result<()> {
+    let context = TestContext::new("3.12");
+
+    let pyproject_toml = context.temp_dir.child("pyproject.toml");
+    pyproject_toml.write_str(
+        r#"
+        [project]
+        name = "project"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = ["typing-extensions"]
+
+        [dependency-groups]
+        foo = ["anyio"]
+        bar = ["iniconfig"]
+        dev = ["sniffio"]
+        "#,
+    )?;
+
+    let test_script = context.temp_dir.child("main.py");
+    test_script.write_str(indoc! { r#"
+        try:
+            import anyio
+            print("imported `anyio`")
+        except ImportError:
+            print("failed to import `anyio`")
+
+        try:
+            import iniconfig
+            print("imported `iniconfig`")
+        except ImportError:
+            print("failed to import `iniconfig`")
+
+        try:
+            import typing_extensions
+            print("imported `typing_extensions`")
+        except ImportError:
+            print("failed to import `typing_extensions`")
+       "#
+    })?;
+
+    context.lock().assert().success();
+
+    uv_snapshot!(context.filters(), context.run().arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    failed to import `anyio`
+    failed to import `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    Resolved 6 packages in [TIME]
+    Prepared 2 packages in [TIME]
+    Installed 2 packages in [TIME]
+     + sniffio==1.3.1
+     + typing-extensions==4.10.0
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--only-group").arg("bar").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    failed to import `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    Resolved 6 packages in [TIME]
+    Prepared 1 package in [TIME]
+    Installed 1 package in [TIME]
+     + iniconfig==2.0.0
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--group").arg("foo").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    imported `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    Resolved 6 packages in [TIME]
+    Prepared 2 packages in [TIME]
+    Installed 2 packages in [TIME]
+     + anyio==4.3.0
+     + idna==3.6
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--group").arg("foo").arg("--group").arg("bar").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    imported `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    Resolved 6 packages in [TIME]
+    Audited 5 packages in [TIME]
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--group").arg("foo").arg("--no-project").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    imported `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    warning: `--group foo` has no effect when used alongside `--no-project`
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--group").arg("foo").arg("--group").arg("bar").arg("--no-project").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    imported `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    warning: `--group` has no effect when used alongside `--no-project`
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--group").arg("dev").arg("--no-project").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    imported `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    warning: `--group dev` has no effect when used alongside `--no-project`
+    "###);
+
+    uv_snapshot!(context.filters(), context.run().arg("--dev").arg("--no-project").arg("main.py"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    imported `anyio`
+    imported `iniconfig`
+    imported `typing_extensions`
+
+    ----- stderr -----
+    warning: `--dev` has no effect when used alongside `--no-project`
+    "###);
+
+    Ok(())
+}
+
 #[test]
 fn run_locked() -> Result<()> {
     let context = TestContext::new("3.12");
@@ -958,6 +1138,62 @@ fn run_locked() -> Result<()> {
 
     let existing = context.read("uv.lock");
 
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            existing, @r###"
+        version = 1
+        requires-python = ">=3.12"
+
+        [options]
+        exclude-newer = "2024-03-25T00:00:00Z"
+
+        [[package]]
+        name = "anyio"
+        version = "3.7.0"
+        source = { registry = "https://pypi.org/simple" }
+        dependencies = [
+            { name = "idna" },
+            { name = "sniffio" },
+        ]
+        sdist = { url = "https://files.pythonhosted.org/packages/c6/b3/fefbf7e78ab3b805dec67d698dc18dd505af7a18a8dd08868c9b4fa736b5/anyio-3.7.0.tar.gz", hash = "sha256:275d9973793619a5374e1c89a4f4ad3f4b0a5510a2b5b939444bee8f4c4d37ce", size = 142737 }
+        wheels = [
+            { url = "https://files.pythonhosted.org/packages/68/fe/7ce1926952c8a403b35029e194555558514b365ad77d75125f521a2bec62/anyio-3.7.0-py3-none-any.whl", hash = "sha256:eddca883c4175f14df8aedce21054bfca3adb70ffe76a9f607aef9d7fa2ea7f0", size = 80873 },
+        ]
+
+        [[package]]
+        name = "idna"
+        version = "3.6"
+        source = { registry = "https://pypi.org/simple" }
+        sdist = { url = "https://files.pythonhosted.org/packages/bf/3f/ea4b9117521a1e9c50344b909be7886dd00a519552724809bb1f486986c2/idna-3.6.tar.gz", hash = "sha256:9ecdbbd083b06798ae1e86adcbfe8ab1479cf864e4ee30fe4e46a003d12491ca", size = 175426 }
+        wheels = [
+            { url = "https://files.pythonhosted.org/packages/c2/e7/a82b05cf63a603df6e68d59ae6a68bf5064484a0718ea5033660af4b54a9/idna-3.6-py3-none-any.whl", hash = "sha256:c05567e9c24a6b9faaa835c4821bad0590fbb9d5779e7caa6e1cc4978e7eb24f", size = 61567 },
+        ]
+
+        [[package]]
+        name = "project"
+        version = "0.1.0"
+        source = { editable = "." }
+        dependencies = [
+            { name = "anyio" },
+        ]
+
+        [package.metadata]
+        requires-dist = [{ name = "anyio", specifier = "==3.7.0" }]
+
+        [[package]]
+        name = "sniffio"
+        version = "1.3.1"
+        source = { registry = "https://pypi.org/simple" }
+        sdist = { url = "https://files.pythonhosted.org/packages/a2/87/a6771e1546d97e7e041b6ae58d80074f81b7d5121207425c964ddf5cfdbd/sniffio-1.3.1.tar.gz", hash = "sha256:f4324edc670a0f49750a81b895f35c3adb843cca46f0530f79fc1babb23789dc", size = 20372 }
+        wheels = [
+            { url = "https://files.pythonhosted.org/packages/e9/44/75a9c9421471a6c4805dbf2356f7c181a29c1879239abab1ea2cc8f38b40/sniffio-1.3.1-py3-none-any.whl", hash = "sha256:2f6da418d1f1e0fddd844478f41680e794e6051915791a034ff65e5f100525a2", size = 10235 },
+        ]
+        "###);
+        }
+    );
+
     // Update the requirements.
     pyproject_toml.write_str(
         r#"
@@ -990,7 +1226,28 @@ fn run_locked() -> Result<()> {
     assert_eq!(existing, updated);
 
     // Lock the updated requirements.
-    context.lock().assert().success();
+    uv_snapshot!(context.lock(), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Resolved 2 packages in [TIME]
+    Removed anyio v3.7.0
+    Removed idna v3.6
+    Added iniconfig v2.0.0
+    Removed sniffio v1.3.1
+    "###);
+
+    // Lock the updated requirements.
+    uv_snapshot!(context.lock(), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Resolved 2 packages in [TIME]
+    "###);
 
     // Running with `--locked` should succeed.
     uv_snapshot!(context.filters(), context.run().arg("--locked").arg("--").arg("python").arg("--version"), @r###"
