@@ -141,6 +141,7 @@ fn tool_run_at_version() {
     The following executables are provided by `pytest`:
     - py.test
     - pytest
+    Consider using `uv tool run --from pytest <EXECUTABLE_NAME>` instead.
 
     ----- stderr -----
     Resolved 4 packages in [TIME]
@@ -202,6 +203,7 @@ fn tool_run_suggest_valid_commands() {
     The following executables are provided by `black`:
     - black
     - blackd
+    Consider using `uv tool run --from black <EXECUTABLE_NAME>` instead.
 
     ----- stderr -----
     Resolved 6 packages in [TIME]
